#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ***************************************************************************
# Copyright 2017-2018 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# ***************************************************************************
import json
import nsx_logging

from cli.commands.cli_command_provider import CliCommandProvider
from cli.utils.location_utils import FilePaths
from cli.utils.node_utils import EXPORT_TYPE_RESTRICTED
from cli.utils.node_utils import get_export_type
from cli.utils.node_utils import get_node_version
from cli.utils.node_utils import is_windows


lg = nsx_logging.getLogger("cli.commands.host_shared.version")
_SUPPORT_URL = "http://www.vmware.com/support.html"


class GetVersion(CliCommandProvider):
    def __call__(self, cmd_list):
        version = get_node_version()
        if not version:
            return "MSG_PUBLIC_CLOUD_VERSION_FAILED"
        res = {"version": version, "support_url": _SUPPORT_URL}
        export_type = get_export_type()
        if export_type and export_type == EXPORT_TYPE_RESTRICTED:
            res["export_type"] = "Limited Export"
        if is_windows():
            # Windows cmd prompt has issues rendering Unicode
            # characters such as the Copyright symbol.
            # Use '(c)' instead of '©' for Windows.
            res["copyright_symbol"] = "(c)"
        return "OK\n\n%s" % json.dumps(res)
