# **********************************************************************
# Copyright 2017 VMware, Inc.  All rights reserved. VMware Confidential.
# **********************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# **********************************************************************
import nsx_logging
import json

from cli.commands.cli_command_provider import CliCommandProvider
from cli.commands.lcp_common.hyperbus.client import get_hyperbus_cli_output
from cli.utils.error_code import ErrorCode as ec

lg = nsx_logging.getLogger("cli.commands.lcp_common.hyperbus.cif_config_cache")

def load_response_output(response):
    try:
        if response is None:
            return None
        output = json.loads(response)
    except (TypeError, ValueError) as err:
        lg.error("load response error: %s", err,
                 error_code=ec.INTERNAL_ERROR)
        return None
    return output

class GetCIFConfigTable(CliCommandProvider):
    def __call__(self, cmd_list):
        response = get_hyperbus_cli_output("cif/table")
        output = load_response_output(response)
        if output is None:
            return "MSG_HOST_CIF_TABLE_FAILED"

        return "OK\n\n%s" % json.dumps(output)

class GetCIFConfigByLSP(CliCommandProvider):
    def __call__(self, cmd_list):
        lspID = cmd_list[3]
        response = get_hyperbus_cli_output("lsp/cif-config", [lspID])
        output = load_response_output(response)
        if output is None:
            return "MSG_HOST_CIF_TABLE_FAILED"

        return "OK\n\n%s" % json.dumps(output)

class GetCIFConfigByAppID(CliCommandProvider):
    def __call__(self, cmd_list):
        appID = cmd_list[3]
        response = get_hyperbus_cli_output("app-id/cif-config", [appID])
        output = load_response_output(response)
        if output is None:
            return "MSG_HOST_CIF_TABLE_FAILED"

        return "OK\n\n%s" % json.dumps(output)

class GetConnectedVIFTable(CliCommandProvider):
    def __call__(self, cmd_list):
        response = get_hyperbus_cli_output("vif/table")
        output = load_response_output(response)
        if output is None:
            return "MSG_HOST_CIF_TABLE_FAILED"

        return "OK\n\n%s" % json.dumps(output)
