# ***************************************************************************
# Copyright 2017-2018 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# ***************************************************************************
import json
import os
import nsx_logging
from cli.utils.command_utils import invoke_command
from cli.utils.constants import NODE_TYPE_EDGE
from cli.utils.constants import NODE_TYPE_HOST_CENTOSKVM
from cli.utils.constants import NODE_TYPE_HOST_SLESKVM
from cli.utils.constants import NODE_TYPE_HOST_ESX
from cli.utils.constants import NODE_TYPE_HOST_RHELKVM
from cli.utils.constants import NODE_TYPE_HOST_UBUNTUKVM
from cli.utils.constants import NODE_TYPE_PUBLIC_CLOUD_GATEWAY
from cli.utils.constants import NODE_TYPE_BAREMETAL_UBUNTU_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_SLES_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_OEL_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_WINDOWS_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_CONTAINER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_CONTAINER
from cli.utils.location_utils import FilePaths
from cli.utils.node_utils import get_node_type
from cli.utils.node_utils import is_windows
from cli.utils.error_code import ErrorCode as ec

lg = nsx_logging.getLogger("cli.commands.lcp_common.opsagent.client")

_OPSAGENT_CLI = {
    NODE_TYPE_EDGE: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        },
    NODE_TYPE_HOST_SLESKVM: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        },
    NODE_TYPE_HOST_CENTOSKVM: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        },
    NODE_TYPE_HOST_ESX: {
        'bin': "/opt/vmware/nsx-cli/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        },
    NODE_TYPE_HOST_RHELKVM: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        },
    NODE_TYPE_HOST_UBUNTUKVM: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        },
    NODE_TYPE_PUBLIC_CLOUD_GATEWAY: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        },
    NODE_TYPE_BAREMETAL_UBUNTU_SERVER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        },
    NODE_TYPE_BAREMETAL_RHEL_SERVER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        },
    NODE_TYPE_BAREMETAL_CENTOS_SERVER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        },
    NODE_TYPE_BAREMETAL_SLES_SERVER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        },
    NODE_TYPE_BAREMETAL_OEL_SERVER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        },
    NODE_TYPE_BAREMETAL_RHEL_CONTAINER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        },
    NODE_TYPE_BAREMETAL_CENTOS_CONTAINER: {
        'bin': "/usr/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-opsagent/opsagent-cli"
        }
    }

if is_windows():
    _OPSAGENT_CLI.update({
        NODE_TYPE_BAREMETAL_WINDOWS_SERVER: {
            'bin': os.path.join(FilePaths.NSX_ROOT, "nsx-cli", "cli", "bin", "nsx-appctl"),
            'socket': os.path.join(FilePaths.NSX_ROOT, "nsx-opsagent", "opsagent-cli")
        }
    })


def has_opsagent_cli(node_type):
    return node_type in (NODE_TYPE_BAREMETAL_CENTOS_CONTAINER,
                         NODE_TYPE_BAREMETAL_CENTOS_SERVER,
                         NODE_TYPE_BAREMETAL_RHEL_CONTAINER,
                         NODE_TYPE_BAREMETAL_RHEL_SERVER,
                         NODE_TYPE_BAREMETAL_UBUNTU_SERVER,
                         NODE_TYPE_BAREMETAL_SLES_SERVER,
                         NODE_TYPE_BAREMETAL_OEL_SERVER,
                         NODE_TYPE_BAREMETAL_WINDOWS_SERVER,
                         NODE_TYPE_EDGE,
                         NODE_TYPE_HOST_CENTOSKVM,
                         NODE_TYPE_HOST_SLESKVM,
                         NODE_TYPE_HOST_ESX,
                         NODE_TYPE_HOST_RHELKVM,
                         NODE_TYPE_HOST_UBUNTUKVM,
                         NODE_TYPE_PUBLIC_CLOUD_GATEWAY)


def load_response_output(response):
    try:
        if response is None:
            return None
        output = json.loads(response)
    except (TypeError, ValueError) as err:
        lg.error("load response error: %s", err)
        return None
    return output


def invoke_opsagent_appctl_command(cmd_list):
    node_type = get_node_type()
    if has_opsagent_cli(node_type):
        cmd = [_OPSAGENT_CLI[node_type]['bin'],
               "-t",
               _OPSAGENT_CLI[node_type]['socket']]
        cmd.extend(cmd_list)
        rc, out, err = invoke_command(cmd)
        return (rc, out, err)
    else:
        lg.error("node type %s does not have opsAgent CLI", node_type,
                 error_code=ec.OPERATION_NOT_SUPPORTED)
        return (-1, None, "invalid command")


def get_opsagent_cli_output(command, argv=None):
    if argv is None:
        argv = []
    cmd_list = [command]

    length = len(argv)
    if length > 0:
        cmd_list.extend(argv)
    rc, out, err = invoke_opsagent_appctl_command(cmd_list)
    if rc != 0:
        lg.error("Failed to invoke opsAgent CLI command, rc: %d, err: %s",
                 rc, err, error_code=ec.OPERATION_FAILED)
        out = None

    return out
