# Copyright (C) 2020 VMware, Inc. All rights reserved.
# -- VMware Confidential

import json
import traceback
import sha.config

from sha.contrib.metric.edge_base import EdgeBase
from sha.contrib.metric.utils import timeout_cache

state_value = {
    "Active": 1,
    "Standby": 2,
    "Sync": 3,
    "Probe": 4,
    "Down": 5}

lr_type_map = {
   'TIER0': 'T0',
   'TIER1': 'T1',
}

class EdgeSrState(EdgeBase):
    def __init__(self):
        super().__init__()
        self._logger = sha.config.get_logger(__name__)

    def get_sr_state(self):
        sr_list_cmd = ['edge-appctl', '-t', '/var/run/vmware/edge/nsxa.ctl',
                       'service-router/show']
        try:
            output = self.run_command_with_time_check(sr_list_cmd) or None
        except Exception as e:
            trace = traceback.format_exc()
            self._logger.warning("Failed to get edge service router: {}, {}".format(e, trace))
            output = None

        sr_list = json.loads(output) if output is not None else None
        new_sr_list = []
        for sr in sr_list:
            if sr is None:
                continue
            if sr['lr_type'] not in ['TIER0', 'TIER1']:
                continue
            runtime = sr["runtime"]
            new_sr_list.append({'uuid': sr['uuid'], 'state': runtime['state'], 'type':lr_type_map[sr['lr_type']]})

        return new_sr_list

    def get_sr_name(self, uuid):
        lr_name_cmd = ['edge-appctl', '-t', '/var/run/vmware/edge/nsxa.ctl',
                       'lrouter/show', '{}'.format(uuid)]

        try:
            output = self.run_command_with_time_check(lr_name_cmd) or None
        except Exception as e:
            trace = traceback.format_exc()
            self._logger.warning("Failed to get edge service router: {}, {}".format(e, trace))
            output = None

        lr = json.loads(output) if output is not None else None
        return lr['name'] if lr else None

    @timeout_cache(10)
    def usage(self, *args, **kwargs):
        states = {}

        sr_list = self.get_sr_state()
        if sr_list is not None:
            for sr in sr_list:
                name = self.get_sr_name(sr['uuid'])
                if name:
                    key = "edge.{}.{}.highavailability_state".format(sr["type"], name)
                else:
                    key = "edge.{}.highavailability_state".format(sr["type"])
                """ If the state is beyond the state-mapper, then return 0, meaning unknown """
                states[key] = state_value.get(sr['state'], 0)
        self._logger.debug("export_data is {}".format(states))

        return states, self.wavefront_source
