# Copyright (C) 2021 VMware, Inc. All rights reserved.
# -- VMware Confidential

profile = data_store['profile']
idps_stats = data_store['metrics']['nsx.edge.idps-stats']

def encap_stats(stats, msg):
    """Edge IDPS Stats"""
    edge_idps_stats = stats.get('edge-idps-stats', None)
    if edge_idps_stats is not None:
        try:
            tmp_msg = nsx_idps_metrics_stats_pb2.NsxIdpsStats(obj_id = "edge-idps-stats",
                      alert_dropped_mp = edge_idps_stats['alert-dropped-mp'],
                      alert_sent_mp = edge_idps_stats['alert-sent-mp'],
                      critical = edge_idps_stats['critical'],
                      dropped_q = edge_idps_stats['dropped-q'],
                      non_critical = edge_idps_stats['non-critical'],
                      proto_dropped_mp = edge_idps_stats['proto-dropped-mp'],
                      proto_sent_mp = edge_idps_stats['proto-sent-mp'],
                      total = edge_idps_stats['total'])
        except KeyError:
            logger.info("Keyerror, not sending IDPS stats to SHA")
        else:
            msg.idps.CopyFrom(tmp_msg)

    """Event Plugin Stats"""
    evt_plugin_stats = stats.get('evt-plugin-stats', None)
    if evt_plugin_stats is not None:
        try:
            tmp_msg = nsx_idps_metrics_stats_pb2.NsxIdpsEvtPluginStats(obj_id = "event-plugin-stats",
                      alert_callback = evt_plugin_stats['AlertCallback'],
                      alert_count = evt_plugin_stats['AlertCount'],
                      alert_sent_count = evt_plugin_stats['AlertSendCount'],
                      big_json_count = evt_plugin_stats['BigJsonCount'],
                      curr_q_count = evt_plugin_stats['CurrQCount'],
                      en_q_fail_count = evt_plugin_stats['EnQFailCount'],
                      evt_oo_mem_count = evt_plugin_stats['EvtOoMemCount'],
                      sock_recnt_count = evt_plugin_stats['SockRecntCount'],
                      sock_send_fail_count = evt_plugin_stats['SockSendFailCount'])
        except KeyError:
            logger.info("Keyerror, not sending IDPS EVT PLUGIN stats to SHA")
        else:
            msg.evt_plugin.CopyFrom(tmp_msg)

    """IDPS Rule Stats"""
    idps_rule_stats = stats.get('idps-rule-stats', None)
    if idps_rule_stats is not None:
        for idps_rule in idps_rule_stats:
            rule_stats = idps_rule.get('stats', None)
            if rule_stats is None:
                continue
            try:
                tmp_msg = nsx_idps_metrics_stats_pb2.NsxIdpsRuleStats(obj_id = str(idps_rule.get('ruleid', 0)),
                          bytes = rule_stats['bytes'],
                          hits = rule_stats['hits'],
                          invocations = rule_stats['invocations'],
                          packets = rule_stats['packets'],
                          sessions = rule_stats['sessions'])
            except KeyError:
                logger.info("Keyerror, not sending IDPS RULE stats to SHA")
            else:
                sub_msg = msg.idps_rule.add()
                sub_msg.CopyFrom(tmp_msg)

def handle_stats(stats):
    entity_type = options_pb2.MetricsEntityType.Value('POLICY_EDGE_NODE')
    msg = metrics_agent_rpc_pb2.MetricsAgentMessage(timestamp = int(idps_stats['timestamp']),
                                                    entity = entity_type,
                                                    entity_id = str(data_store['host_id']))
    encap_stats(stats, msg)
    Exportdata(data = msg, exports = ['tsdb_metrics'])

if idps_stats and idps_stats['value']:
    logger.debug("IDPS_STATS stats is {}".format(idps_stats['value']))
    handle_stats(idps_stats['value'])
else:
    logger.info("NO IDPS_STATS stats for exporting.")
