#!/bin/bash
#
# Copyright (C) 2018 VMware, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-2-Clause
#
#

set -e
LPID="/etc/vmware/nsx-bm/lpid"
get_lp_id()
{
    if [ -f $LPID ]; then
        echo "lpid="`cat $LPID`
    fi
}

BMS_CONF="/etc/vmware/nsx-bm/bms.conf"
get_config_mode()
{
    if [ -f $BMS_CONF ]; then
        if grep -q "config_mode" $BMS_CONF; then
            cat $BMS_CONF
            echo
        else
            echo -n "config_mode="
            cat $BMS_CONF
        fi
    fi
}

BMS_STARTUP_SCRIPT="/etc/init.d/nsx-baremetal"

get_vif_id()
{
    echo "app_vif_id="`grep "external-ids:iface-id=" $BMS_STARTUP_SCRIPT | sed -r 's/.*external-ids:iface-id=(\w{8}(-\w{4}){3}-\w{12}).*/\1/g'`
}

get_mac_address()
{
    echo "mac_address="`grep "external-ids:attached-mac=" $BMS_STARTUP_SCRIPT | sed -r 's/.*external-ids:attached-mac=((\w{2}:){5}\w{2}).*/\1/g'`
}

get_static_address()
{
    local ip=`grep "ifconfig .* netmask .* up" $BMS_STARTUP_SCRIPT | sed -r 's/.*ifconfig .* ((([0-9]{1,3}\.){3}([0-9]{1,3}))) netmask .*/\1/g'`
    local netmask=`grep "ifconfig .* netmask .* up" $BMS_STARTUP_SCRIPT | sed -r 's/.*ifconfig .* netmask ((([0-9]{1,3}\.){3}([0-9]{1,3}))).*/\1/g'`
    echo "static_address="$ip/`ipcalc -p 1.1.1.1 $netmask | sed -r 's/^PREFIX=(.*)/\1/g'`
}

get_routing_rules()
{
    grep "route add .* dev .*" $BMS_STARTUP_SCRIPT | sed -r 's/.*route add (.*) dev .*/\1/' |
    while read -r rule
    do
        echo "routing_rule="$rule
    done
}

get_lp_id
get_config_mode
get_vif_id
get_mac_address
if grep -q "route add .* dev .*" $BMS_STARTUP_SCRIPT; then
    get_routing_rules
fi
if [ "x`cat $BMS_CONF`" = "xstatic" ] || grep -q "config_mode=static" $BMS_CONF ; then
    get_static_address
fi
